searchData={"items":[{"type":"module","title":"ExWal","doc":"**ExWal is a project that aims to provide a solution for managing write-ahead log (WAL) in Elixir.**","ref":"ExWal.html"},{"type":"module","title":"Installation - ExWal","doc":"The package can be installed by adding `ex_wal` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_wal, \"~> 0.2\"}\n  ]\nend\n```","ref":"ExWal.html#module-installation"},{"type":"module","title":"Design - ExWal","doc":"The design of this project borrows from the implementation of WAL in the [Pebble](https://github.com/cockroachdb/pebble) project.","ref":"ExWal.html#module-design"},{"type":"module","title":"Usage - ExWal","doc":"1. Prepare a instance\n```Elixir\ndefmodule MyApp do\n  use ExWal, otp_app: :my_app\nend\n```\n\n2. Choose your file system implementation\nExWal provides 2 file system implementations:\n- `ExWal.FS.Default`\n- `ExWal.FS.Syncing`\n\n`ExWal.FS.Syncing` provider better write performance. You can use `MyApp.syncing_fs/0` to get a `ExWal.FS.Syncing` instance.\n\n\n3. Add it to supervised tree\n```elixir\nSupervisor.start_link(\n  [\n    MyApp\n  ],\n  strategy: :one_for_one\n)\n```\n\n4. Get a manager\n\nExWal provides 2 manager implementations:\n- `ExWal.Manager.Standalone`\n- `ExWal.Manager.Failover`\n\nYou can use `MyApp.manager/3` to get a `ExWal.Manager` instance.\n\n```Elixir\n# get a standalone manager\n{:ok, m} =\n      MyApp.manager(:standalone, \"my-manager\", %ExWal.Manager.Options{\n        primary: [\n          fs: MyApp.syncing_fs(),\n          dir: \"my-primary-dir-path\"\n        ]\n      })\n\n# get a failover manager\n{:ok, m} =\n      MyApp.manager(:failover, \"my-manager\", %ExWal.Manager.Options{\n        primary: [\n          fs: MyApp.syncing_fs(),\n          dir: \"my-primary-dir-path\"\n        ],\n        secondary: [\n          fs: MyApp.syncing_fs(),\n          dir: \"my-secondary-dir-path\"\n        ]\n      })\n```\n\n5. Create a WAL writer\n\nManager provides a `create/2` function to create a WAL writer. Writer is a instance which implements `ExWal.LogWriter` protocol.\n\n```Elixir\n{:ok, m} =\n      MyApp.manager(:standalone, \"my-manager\", %ExWal.Manager.Options{\n        primary: [\n          fs: MyApp.syncing_fs(),\n          dir: \"my-primary-dir-path\"\n        ]\n      })\n\n{:ok, writer} = ExWal.Manager.create(m, 1)\n\n1..50\n|> Enum.map(fn x ->\n  s =\n    x\n    |> Integer.to_string()\n    |> String.pad_leading(4, \"0\")\n\n  \"Hello Elixir! I am a developer. I love Elixir #{s}.\"\nend)\n|> Enum.each(fn data -> LogWriter.write_record(writer, data) end)\n```\n\n6. Create a WAL reader\n\nManager provides a `list/2` function to list all WAL files. You can use `MyApp.open_for_read/1` function to create a Reader.  Reader is a instance which implements `ExWal.LogReader` protocol.\n\n```Elixir\n{:ok, m} =\n      MyApp.manager(:standalone, \"my-manager\", %ExWal.Manager.Options{\n        primary: [\n          fs: MyApp.syncing_fs(),\n          dir: \"my-primary-dir-path\"\n        ]\n      })\n\n{:ok, [log | _]} = ExWal.Manager.list(m)\n{:ok, reader} = MyApp.open_for_read(log)\n\nreader\n|> Enum.reduce_while(fn reader ->\n  case LogReader.next(reader) do\n    :eof ->\n      {:halt, :ok}\n\n    {:error, _reason} ->\n      LogReader.recovery(reader)\n      {:cont, reader}\n\n    # raise ExWal.Exception, message: \"read failed: #{inspect(reason)}\"\n\n    bin ->\n      IO.puts(bin)\n      {:cont, reader}\n  end\nend)\n```","ref":"ExWal.html#module-usage"},{"type":"module","title":"Benchmark - ExWal","doc":"See [benchmarks/report.md](benchmarks/report.md) for more details.\n\n-----\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"ExWal.html#module-benchmark"},{"type":"module","title":"ExWal.Constant","doc":"This module provides a macro for defining compile-time constants in Elixir code.","ref":"ExWal.Constant.html"},{"type":"module","title":"Examples - ExWal.Constant","doc":"```Elixir\n\n# Define a constant:\ndefmodule MyConst do\n  import Constants\n\n  const :pi, 3.14159\nend\n\n# Use a constant:\n\nrequire MyConst\n@pi MyConst.pi()\n\nIO.puts(@pi)\n```","ref":"ExWal.Constant.html#module-examples"},{"type":"macro","title":"ExWal.Constant.const/2","doc":"","ref":"ExWal.Constant.html#const/2"},{"type":"module","title":"ExWal.Debug","doc":"Debug tools","ref":"ExWal.Debug.html"},{"type":"function","title":"ExWal.Debug.debug/1","doc":"","ref":"ExWal.Debug.html#debug/1"},{"type":"function","title":"ExWal.Debug.stacktrace/1","doc":"","ref":"ExWal.Debug.html#stacktrace/1"},{"type":"protocol","title":"ExWal.FS","doc":"","ref":"ExWal.FS.html"},{"type":"function","title":"ExWal.FS.create/2","doc":"Create creates the named file for reading and writing. If a file\nalready exists at the provided name, it's removed first ensuring the\nresulting file descriptor points to a new inode.","ref":"ExWal.FS.html#create/2"},{"type":"function","title":"ExWal.FS.link/3","doc":"create_link creates a hard link to the file","ref":"ExWal.FS.html#link/3"},{"type":"function","title":"ExWal.FS.list/2","doc":"list lists the files in the given directory.","ref":"ExWal.FS.html#list/2"},{"type":"function","title":"ExWal.FS.lock/2","doc":"Lock locks the given file, creating the file if necessary, and\ntruncating the file if it already exists. The lock is an exclusive lock\n(a write lock), but locked files should neither be read from nor written\nto. Such files should have zero size and only exist to co-ordinate\nownership across processes.\n\nA nil Closer is returned if an error occurred. Otherwise, close that\nCloser to release the lock.\n\nOn Linux and OSX, a lock has the same semantics as fcntl(2)'s advisory\nlocks. In particular, closing any other file descriptor for the same\nfile will release the lock prematurely.\n\nAttempting to lock a file that is already locked by the current process\nreturns an error and leaves the existing lock untouched.\n\nLock is not yet implemented on other operating systems, and calling it","ref":"ExWal.FS.html#lock/2"},{"type":"function","title":"ExWal.FS.mkdir_all/2","doc":"mkdir_all creates a directory and any necessary parents. The permission bits\nperm have the same semantics as for os.mkdir. If the directory already exists,\nmkdir_all does nothing.","ref":"ExWal.FS.html#mkdir_all/2"},{"type":"function","title":"ExWal.FS.open/3","doc":"open opens the named file for reading.","ref":"ExWal.FS.html#open/3"},{"type":"function","title":"ExWal.FS.open_dir/2","doc":"open_dir opens the named directory for syncing","ref":"ExWal.FS.html#open_dir/2"},{"type":"function","title":"ExWal.FS.open_read_write/3","doc":"open_read_write opens the named file for reading and writing. If the file\ndoes not exist, it is created.","ref":"ExWal.FS.html#open_read_write/3"},{"type":"function","title":"ExWal.FS.remove/2","doc":"remove a file","ref":"ExWal.FS.html#remove/2"},{"type":"function","title":"ExWal.FS.remove_all/2","doc":"remove_all removes all files in a directory","ref":"ExWal.FS.html#remove_all/2"},{"type":"function","title":"ExWal.FS.rename/3","doc":"rename renames a file","ref":"ExWal.FS.html#rename/3"},{"type":"function","title":"ExWal.FS.reuse_for_write/3","doc":"reuse_for_write attempts to reuse the file with oldname by renaming it to newname and opening\nit for writing without truncation. It is acceptable for the implementation to choose not\nto reuse oldname, and simply create the file with newname -- in this case the implementation\nshould delete oldname. If the caller calls this function with an oldname that does not exist,\nthe implementation may return an error.","ref":"ExWal.FS.html#reuse_for_write/3"},{"type":"function","title":"ExWal.FS.stat/2","doc":"stat returns the file info for the given file.","ref":"ExWal.FS.html#stat/2"},{"type":"type","title":"ExWal.FS.file/0","doc":"","ref":"ExWal.FS.html#t:file/0"},{"type":"type","title":"ExWal.FS.t/0","doc":"All the types that implement this protocol.","ref":"ExWal.FS.html#t:t/0"},{"type":"module","title":"ExWal.FS.Default","doc":"Default implementation of the `ExWal.FS` protocols.","ref":"ExWal.FS.Default.html"},{"type":"module","title":"ExWal.FS.Syncing","doc":"Syncing file system. This implementation create *syncing* file which provider better write performance.","ref":"ExWal.FS.Syncing.html"},{"type":"function","title":"ExWal.FS.Syncing.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExWal.FS.Syncing.html#child_spec/1"},{"type":"function","title":"ExWal.FS.Syncing.create/2","doc":"","ref":"ExWal.FS.Syncing.html#create/2"},{"type":"function","title":"ExWal.FS.Syncing.handle_create/2","doc":"","ref":"ExWal.FS.Syncing.html#handle_create/2"},{"type":"function","title":"ExWal.FS.Syncing.handle_open_read_write/2","doc":"","ref":"ExWal.FS.Syncing.html#handle_open_read_write/2"},{"type":"function","title":"ExWal.FS.Syncing.init/4","doc":"","ref":"ExWal.FS.Syncing.html#init/4"},{"type":"function","title":"ExWal.FS.Syncing.open_read_write/2","doc":"","ref":"ExWal.FS.Syncing.html#open_read_write/2"},{"type":"function","title":"ExWal.FS.Syncing.start_link/1","doc":"","ref":"ExWal.FS.Syncing.html#start_link/1"},{"type":"type","title":"ExWal.FS.Syncing.t/0","doc":"","ref":"ExWal.FS.Syncing.html#t:t/0"},{"type":"protocol","title":"ExWal.File","doc":"File protocol","ref":"ExWal.File.html"},{"type":"function","title":"ExWal.File.close/1","doc":"","ref":"ExWal.File.html#close/1"},{"type":"function","title":"ExWal.File.fd/1","doc":"fd returns the raw file descriptor when a File is backed by an :file.\nIt can be used for specific functionality like Prefetch.\nReturns error if not supported.","ref":"ExWal.File.html#fd/1"},{"type":"function","title":"ExWal.File.prealloc/3","doc":"prealloc optionally preallocates storage for `length` at `offset`\nwithin the file. Implementations may choose to do nothing.","ref":"ExWal.File.html#prealloc/3"},{"type":"function","title":"ExWal.File.prefetch/3","doc":"prefetch signals the OS (on supported platforms) to fetch the next length\nbytes in file after offset into cache. Any\nsubsequent reads in that range will not issue disk IO.","ref":"ExWal.File.html#prefetch/3"},{"type":"function","title":"ExWal.File.read/2","doc":"read reads n bytes from the file","ref":"ExWal.File.html#read/2"},{"type":"function","title":"ExWal.File.read_at/3","doc":"read_at reads n bytes from the file at the given offset of datasource","ref":"ExWal.File.html#read_at/3"},{"type":"function","title":"ExWal.File.stat/1","doc":"","ref":"ExWal.File.html#stat/1"},{"type":"function","title":"ExWal.File.sync/1","doc":"","ref":"ExWal.File.html#sync/1"},{"type":"function","title":"ExWal.File.sync_to/2","doc":"sync_to requests that a prefix of the file's data be synced to stable\nstorage. The caller passes provides a `length`, indicating how many bytes\nto sync from the beginning of the file. sync_to is a no-op for\ndirectories, and therefore always returns false.","ref":"ExWal.File.html#sync_to/2"},{"type":"function","title":"ExWal.File.write/2","doc":"write writes bytes to the file","ref":"ExWal.File.html#write/2"},{"type":"function","title":"ExWal.File.write_at/3","doc":"write_at writes bytes to the file to the underlying data stream","ref":"ExWal.File.html#write_at/3"},{"type":"type","title":"ExWal.File.t/0","doc":"All the types that implement this protocol.","ref":"ExWal.File.html#t:t/0"},{"type":"module","title":"ExWal.File.Syncing","doc":"Syncing file implementation. This implementation do syncing work automatically after writting.\nSee `may_sync` function for specific synchronization conditions","ref":"ExWal.File.Syncing.html"},{"type":"function","title":"ExWal.File.Syncing.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExWal.File.Syncing.html#child_spec/1"},{"type":"function","title":"ExWal.File.Syncing.close/1","doc":"","ref":"ExWal.File.Syncing.html#close/1"},{"type":"function","title":"ExWal.File.Syncing.get/1","doc":"","ref":"ExWal.File.Syncing.html#get/1"},{"type":"function","title":"ExWal.File.Syncing.handle_close/1","doc":"","ref":"ExWal.File.Syncing.html#handle_close/1"},{"type":"function","title":"ExWal.File.Syncing.handle_write/2","doc":"","ref":"ExWal.File.Syncing.html#handle_write/2"},{"type":"function","title":"ExWal.File.Syncing.init/2","doc":"","ref":"ExWal.File.Syncing.html#init/2"},{"type":"function","title":"ExWal.File.Syncing.start_link/1","doc":"","ref":"ExWal.File.Syncing.html#start_link/1"},{"type":"function","title":"ExWal.File.Syncing.write/2","doc":"","ref":"ExWal.File.Syncing.html#write/2"},{"type":"type","title":"ExWal.File.Syncing.t/0","doc":"","ref":"ExWal.File.Syncing.html#t:t/0"},{"type":"protocol","title":"ExWal.LogReader","doc":"A LogReader takes an ordered sequence of physical WAL files\n(\"segments\"), providing a merged view of the WAL's logical contents.\nIt's responsible for filtering duplicate records which may be shared by the tail of a segment file and the head of its\nsuccessor.","ref":"ExWal.LogReader.html"},{"type":"function","title":"ExWal.LogReader.next/1","doc":"next returns a next binary written to wal.\nIt returns io.EOF if there are no more records.","ref":"ExWal.LogReader.html#next/1"},{"type":"function","title":"ExWal.LogReader.recovery/1","doc":"recovery clear the rest part of current block and move to next block.","ref":"ExWal.LogReader.html#recovery/1"},{"type":"function","title":"ExWal.LogReader.stop/1","doc":"","ref":"ExWal.LogReader.html#stop/1"},{"type":"type","title":"ExWal.LogReader.t/0","doc":"All the types that implement this protocol.","ref":"ExWal.LogReader.html#t:t/0"},{"type":"module","title":"ExWal.LogReader.Virtual","doc":"A virtual log consists of a set of segments. Each segment is a physical WAL file.","ref":"ExWal.LogReader.Virtual.html"},{"type":"function","title":"ExWal.LogReader.Virtual.get/1","doc":"","ref":"ExWal.LogReader.Virtual.html#get/1"},{"type":"function","title":"ExWal.LogReader.Virtual.handle_closed/1","doc":"","ref":"ExWal.LogReader.Virtual.html#handle_closed/1"},{"type":"function","title":"ExWal.LogReader.Virtual.handle_next/1","doc":"","ref":"ExWal.LogReader.Virtual.html#handle_next/1"},{"type":"function","title":"ExWal.LogReader.Virtual.handle_recovery/1","doc":"","ref":"ExWal.LogReader.Virtual.html#handle_recovery/1"},{"type":"function","title":"ExWal.LogReader.Virtual.init/3","doc":"","ref":"ExWal.LogReader.Virtual.html#init/3"},{"type":"function","title":"ExWal.LogReader.Virtual.next/1","doc":"","ref":"ExWal.LogReader.Virtual.html#next/1"},{"type":"function","title":"ExWal.LogReader.Virtual.recovery/1","doc":"","ref":"ExWal.LogReader.Virtual.html#recovery/1"},{"type":"function","title":"ExWal.LogReader.Virtual.start_link/1","doc":"","ref":"ExWal.LogReader.Virtual.html#start_link/1"},{"type":"function","title":"ExWal.LogReader.Virtual.stop/1","doc":"","ref":"ExWal.LogReader.Virtual.html#stop/1"},{"type":"type","title":"ExWal.LogReader.Virtual.t/0","doc":"","ref":"ExWal.LogReader.Virtual.html#t:t/0"},{"type":"protocol","title":"ExWal.LogWriter","doc":"Writer writes to a virtual WAL. A Writer in standalone mode maps to a\nsingle LogWriter. In failover mode, it can failover across multiple\nphysical log files.","ref":"ExWal.LogWriter.html"},{"type":"function","title":"ExWal.LogWriter.stop/1","doc":"stop stops the writer.","ref":"ExWal.LogWriter.html#stop/1"},{"type":"function","title":"ExWal.LogWriter.write_record/3","doc":"write_record writes a complete record. The record is asynchronously persisted to the underlying writer.","ref":"ExWal.LogWriter.html#write_record/3"},{"type":"type","title":"ExWal.LogWriter.t/0","doc":"All the types that implement this protocol.","ref":"ExWal.LogWriter.html#t:t/0"},{"type":"module","title":"ExWal.LogWriter.Failover","doc":"FailoverWriter is the implementation of LogWriter in failover mode.","ref":"ExWal.LogWriter.Failover.html"},{"type":"function","title":"ExWal.LogWriter.Failover.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExWal.LogWriter.Failover.html#child_spec/1"},{"type":"function","title":"ExWal.LogWriter.Failover.current_observer/1","doc":"","ref":"ExWal.LogWriter.Failover.html#current_observer/1"},{"type":"function","title":"ExWal.LogWriter.Failover.get/1","doc":"","ref":"ExWal.LogWriter.Failover.html#get/1"},{"type":"function","title":"ExWal.LogWriter.Failover.get_log/1","doc":"","ref":"ExWal.LogWriter.Failover.html#get_log/1"},{"type":"function","title":"ExWal.LogWriter.Failover.start_link/1","doc":"","ref":"ExWal.LogWriter.Failover.html#start_link/1"},{"type":"function","title":"ExWal.LogWriter.Failover.stop/1","doc":"","ref":"ExWal.LogWriter.Failover.html#stop/1"},{"type":"function","title":"ExWal.LogWriter.Failover.switch_dir/2","doc":"","ref":"ExWal.LogWriter.Failover.html#switch_dir/2"},{"type":"function","title":"ExWal.LogWriter.Failover.write_record/3","doc":"","ref":"ExWal.LogWriter.Failover.html#write_record/3"},{"type":"type","title":"ExWal.LogWriter.Failover.t/0","doc":"","ref":"ExWal.LogWriter.Failover.html#t:t/0"},{"type":"module","title":"ExWal.LogWriter.Single","doc":"log writer writes records to a WAL file.","ref":"ExWal.LogWriter.Single.html"},{"type":"function","title":"ExWal.LogWriter.Single.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExWal.LogWriter.Single.html#child_spec/1"},{"type":"function","title":"ExWal.LogWriter.Single.get/1","doc":"","ref":"ExWal.LogWriter.Single.html#get/1"},{"type":"function","title":"ExWal.LogWriter.Single.start_link/1","doc":"","ref":"ExWal.LogWriter.Single.html#start_link/1"},{"type":"function","title":"ExWal.LogWriter.Single.stop/1","doc":"","ref":"ExWal.LogWriter.Single.html#stop/1"},{"type":"function","title":"ExWal.LogWriter.Single.write_record/3","doc":"","ref":"ExWal.LogWriter.Single.html#write_record/3"},{"type":"type","title":"ExWal.LogWriter.Single.t/0","doc":"","ref":"ExWal.LogWriter.Single.html#t:t/0"},{"type":"protocol","title":"ExWal.Manager","doc":"The manager protocol for the WAL.","ref":"ExWal.Manager.html"},{"type":"function","title":"ExWal.Manager.close/1","doc":"","ref":"ExWal.Manager.html#close/1"},{"type":"function","title":"ExWal.Manager.create/2","doc":"","ref":"ExWal.Manager.html#create/2"},{"type":"function","title":"ExWal.Manager.list/1","doc":"","ref":"ExWal.Manager.html#list/1"},{"type":"function","title":"ExWal.Manager.obsolete/3","doc":"","ref":"ExWal.Manager.html#obsolete/3"},{"type":"type","title":"ExWal.Manager.t/0","doc":"All the types that implement this protocol.","ref":"ExWal.Manager.html#t:t/0"},{"type":"module","title":"ExWal.Models.Block","doc":"A block of data in a WAL file.","ref":"ExWal.Models.Block.html"},{"type":"function","title":"ExWal.Models.Block.append/2","doc":"","ref":"ExWal.Models.Block.html#append/2"},{"type":"function","title":"ExWal.Models.Block.flush_all/1","doc":"","ref":"ExWal.Models.Block.html#flush_all/1"},{"type":"function","title":"ExWal.Models.Block.flush_to/2","doc":"","ref":"ExWal.Models.Block.html#flush_to/2"},{"type":"function","title":"ExWal.Models.Block.flushable/1","doc":"","ref":"ExWal.Models.Block.html#flushable/1"},{"type":"function","title":"ExWal.Models.Block.fullfill/2","doc":"","ref":"ExWal.Models.Block.html#fullfill/2"},{"type":"function","title":"ExWal.Models.Block.new/0","doc":"","ref":"ExWal.Models.Block.html#new/0"},{"type":"type","title":"ExWal.Models.Block.t/0","doc":"","ref":"ExWal.Models.Block.html#t:t/0"},{"type":"module","title":"ExWal.Models.LegacyRecord","doc":"The wire format is that the stream is divided into 32KiB blocks, and each\nblock contains a number of tightly packed chunks. Chunks cannot cross block\nboundaries. The last block may be shorter than 32 KiB. Any unused bytes in a\nblock must be zero.\n\nA record maps to one or more chunks. There are two chunk formats: legacy and\nrecyclable. The legacy chunk format:\n\n+----------+-----------+-----------+--- ... ---+\n| CRC (4B) | Size (2B) | Type (1B) | Payload   |\n+----------+-----------+-----------+--- ... ---+\n\nCRC is computed over the type and payload\nSize is the length of the payload in bytes\nType is the chunk type\n\nThere are four chunk types: whether the chunk is the full record, or the\nfirst, middle or last chunk of a multi-chunk record. A multi-chunk record\nhas one first chunk, zero or more middle chunks, and one last chunk.","ref":"ExWal.Models.LegacyRecord.html"},{"type":"function","title":"ExWal.Models.LegacyRecord.encode/1","doc":"","ref":"ExWal.Models.LegacyRecord.html#encode/1"},{"type":"type","title":"ExWal.Models.LegacyRecord.t/0","doc":"","ref":"ExWal.Models.LegacyRecord.html#t:t/0"},{"type":"module","title":"ExWal.Models.RecyclableRecord","doc":"The recyclyable chunk format is similar to the legacy format, but extends\nthe chunk header with an additional log number field. This allows reuse\n(recycling) of log files which can provide significantly better performance\nwhen syncing frequently as it avoids needing to update the file\nmetadata. Additionally, recycling log files is a prequisite for using direct\nIO with log writing. The recyclyable format is:\n\n+----------+-----------+-----------+----------------+--- ... ---+\n| CRC (4B) | Size (2B) | Type (1B) | Log number (4B)| Payload   |\n+----------+-----------+-----------+----------------+--- ... ---+\n\nRecyclable chunks are distinguished from legacy chunks by the addition of 4\nextra \"recyclable\" chunk types that map directly to the legacy chunk types\n(i.e. full, first, middle, last). The CRC is computed over the type, log\nnumber, and payload.\n\nThe wire format allows for limited recovery in the face of data corruption:\non a format error (such as a checksum mismatch), the reader moves to the\nnext block and looks for the next full or first chunk.","ref":"ExWal.Models.RecyclableRecord.html"},{"type":"function","title":"ExWal.Models.RecyclableRecord.empty?/1","doc":"","ref":"ExWal.Models.RecyclableRecord.html#empty?/1"},{"type":"function","title":"ExWal.Models.RecyclableRecord.encode/1","doc":"","ref":"ExWal.Models.RecyclableRecord.html#encode/1"},{"type":"function","title":"ExWal.Models.RecyclableRecord.last_chunk?/1","doc":"","ref":"ExWal.Models.RecyclableRecord.html#last_chunk?/1"},{"type":"function","title":"ExWal.Models.RecyclableRecord.new/3","doc":"","ref":"ExWal.Models.RecyclableRecord.html#new/3"},{"type":"function","title":"ExWal.Models.RecyclableRecord.parse/1","doc":"","ref":"ExWal.Models.RecyclableRecord.html#parse/1"},{"type":"type","title":"ExWal.Models.RecyclableRecord.t/0","doc":"","ref":"ExWal.Models.RecyclableRecord.html#t:t/0"},{"type":"protocol","title":"ExWal.Recycler","doc":"The Recycler protocol defines the behaviour of a WAL file recycler.","ref":"ExWal.Recycler.html"},{"type":"function","title":"ExWal.Recycler.add/2","doc":"","ref":"ExWal.Recycler.html#add/2"},{"type":"function","title":"ExWal.Recycler.get_min/1","doc":"get_min returns the current minimum log number that is allowed to be recycled.","ref":"ExWal.Recycler.html#get_min/1"},{"type":"function","title":"ExWal.Recycler.initialize/2","doc":"Add attempts to recycle the log file specified by file. Returns true if\nthe log file should not be deleted (i.e. the log is being recycled), and\nfalse otherwise.","ref":"ExWal.Recycler.html#initialize/2"},{"type":"function","title":"ExWal.Recycler.peek/1","doc":"Peek returns the log at the head of the recycling queue, or the zero value\nfileInfo and false if the queue is empty.","ref":"ExWal.Recycler.html#peek/1"},{"type":"function","title":"ExWal.Recycler.pop/1","doc":"Pop removes the log number at the head of the recycling queue, enforcing\nthat it matches the specified seq. An error is returned of the recycling\nqueue is empty or the head log number does not match the specified one.","ref":"ExWal.Recycler.html#pop/1"},{"type":"function","title":"ExWal.Recycler.set_min/2","doc":"set_min sets the minimum log number that is allowed to be recycled.","ref":"ExWal.Recycler.html#set_min/2"},{"type":"function","title":"ExWal.Recycler.stop/1","doc":"","ref":"ExWal.Recycler.html#stop/1"},{"type":"type","title":"ExWal.Recycler.log/0","doc":"","ref":"ExWal.Recycler.html#t:log/0"},{"type":"type","title":"ExWal.Recycler.log_num/0","doc":"","ref":"ExWal.Recycler.html#t:log_num/0"},{"type":"type","title":"ExWal.Recycler.t/0","doc":"All the types that implement this protocol.","ref":"ExWal.Recycler.html#t:t/0"},{"type":"module","title":"ExWal.Recycler.ETS","doc":"An ETS based WAL file recycler.","ref":"ExWal.Recycler.ETS.html"},{"type":"function","title":"ExWal.Recycler.ETS.add/2","doc":"","ref":"ExWal.Recycler.ETS.html#add/2"},{"type":"function","title":"ExWal.Recycler.ETS.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExWal.Recycler.ETS.html#child_spec/1"},{"type":"function","title":"ExWal.Recycler.ETS.get/1","doc":"","ref":"ExWal.Recycler.ETS.html#get/1"},{"type":"function","title":"ExWal.Recycler.ETS.get_min/1","doc":"","ref":"ExWal.Recycler.ETS.html#get_min/1"},{"type":"function","title":"ExWal.Recycler.ETS.handle_add/2","doc":"","ref":"ExWal.Recycler.ETS.html#handle_add/2"},{"type":"function","title":"ExWal.Recycler.ETS.handle_get_min/1","doc":"","ref":"ExWal.Recycler.ETS.html#handle_get_min/1"},{"type":"function","title":"ExWal.Recycler.ETS.handle_initialize/2","doc":"","ref":"ExWal.Recycler.ETS.html#handle_initialize/2"},{"type":"function","title":"ExWal.Recycler.ETS.handle_peek/1","doc":"","ref":"ExWal.Recycler.ETS.html#handle_peek/1"},{"type":"function","title":"ExWal.Recycler.ETS.handle_pop/1","doc":"","ref":"ExWal.Recycler.ETS.html#handle_pop/1"},{"type":"function","title":"ExWal.Recycler.ETS.handle_set_min/2","doc":"","ref":"ExWal.Recycler.ETS.html#handle_set_min/2"},{"type":"function","title":"ExWal.Recycler.ETS.init/1","doc":"","ref":"ExWal.Recycler.ETS.html#init/1"},{"type":"function","title":"ExWal.Recycler.ETS.initialize/2","doc":"","ref":"ExWal.Recycler.ETS.html#initialize/2"},{"type":"function","title":"ExWal.Recycler.ETS.peek/1","doc":"","ref":"ExWal.Recycler.ETS.html#peek/1"},{"type":"function","title":"ExWal.Recycler.ETS.pop/1","doc":"","ref":"ExWal.Recycler.ETS.html#pop/1"},{"type":"function","title":"ExWal.Recycler.ETS.set_min/2","doc":"","ref":"ExWal.Recycler.ETS.html#set_min/2"},{"type":"function","title":"ExWal.Recycler.ETS.start_link/1","doc":"","ref":"ExWal.Recycler.ETS.html#start_link/1"},{"type":"function","title":"ExWal.Recycler.ETS.stop/1","doc":"","ref":"ExWal.Recycler.ETS.html#stop/1"},{"type":"type","title":"ExWal.Recycler.ETS.log/0","doc":"","ref":"ExWal.Recycler.ETS.html#t:log/0"},{"type":"type","title":"ExWal.Recycler.ETS.log_num/0","doc":"","ref":"ExWal.Recycler.ETS.html#t:log_num/0"},{"type":"type","title":"ExWal.Recycler.ETS.p/0","doc":"","ref":"ExWal.Recycler.ETS.html#t:p/0"},{"type":"type","title":"ExWal.Recycler.ETS.t/0","doc":"","ref":"ExWal.Recycler.ETS.html#t:t/0"},{"type":"module","title":"ExWal.Typespecs","doc":"Common typespecs for ExWal","ref":"ExWal.Typespecs.html"},{"type":"type","title":"ExWal.Typespecs.dict/0","doc":"","ref":"ExWal.Typespecs.html#t:dict/0"},{"type":"type","title":"ExWal.Typespecs.index/0","doc":"","ref":"ExWal.Typespecs.html#t:index/0"},{"type":"type","title":"ExWal.Typespecs.name/0","doc":"","ref":"ExWal.Typespecs.html#t:name/0"},{"type":"type","title":"ExWal.Typespecs.on_start/0","doc":"","ref":"ExWal.Typespecs.html#t:on_start/0"},{"type":"type","title":"ExWal.Typespecs.opts/0","doc":"","ref":"ExWal.Typespecs.html#t:opts/0"},{"type":"type","title":"ExWal.Typespecs.permision/0","doc":"","ref":"ExWal.Typespecs.html#t:permision/0"},{"type":"exception","title":"ExWal.Exception","doc":"General exception with an optional string, map, or Keyword list stored\nin exception details","ref":"ExWal.Exception.html"},{"type":"function","title":"ExWal.Exception.new/2","doc":"","ref":"ExWal.Exception.html#new/2"},{"type":"type","title":"ExWal.Exception.t/0","doc":"","ref":"ExWal.Exception.html#t:t/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}